package de.fhdw.gaming.ipspiel23.tictactoe.core.evaluation;

import java.util.Optional;
import java.util.Set;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToeFieldState;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToePlayer;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToeRow;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToeState;

/**
 * A win evaluation strategy for evaluating the state of a Tic-Tac-Toe game.
 * This strategy focuses on detecting if there are any rows on the board that are uniformly marked by a player.
 * If such rows exist, a predefined score is assigned to the game state based on the marking player.
 * 
 * This strategy assigns a high positive score if the current player has uniformly marked rows,
 * and a high negative score if the opponent player has uniformly marked rows.
 * 
 * @see IEvaluationStrategy
 * @see TicTacToeState
 *
 */
public class WinEvaluationStrategy implements IEvaluationStrategy {

    /**
     * Evaluates the given Tic-Tac-Toe game state using the win evaluation strategy.
     * This strategy focuses on detecting uniformly marked rows on the board.
     * If uniformly marked rows exist, a predefined score is assigned based on the marking player.
     * 
     * Too complex, but not easily simplifyable. Warning suppressed.
     * 
     * @param state The Tic-Tac-Toe game state to be evaluated.
     * @return An optional containing the evaluation score as an integer value.
     */
    @Override
    @SuppressWarnings({ "checkstyle:CyclomaticComplexity", "PMD.CyclomaticComplexity" })
    public Optional<Integer> evaluateState(final TicTacToeState state) {
        final Set<TicTacToeRow> fullRows = state.getBoard().getRowsUniformlyMarked();

        if (fullRows.isEmpty()) {

            return Optional.of(0);
        }
        
        Integer returnValue = 0;
        final TicTacToePlayer currentPlayer = state.getCurrentPlayer();
        
        for (final TicTacToeRow r : fullRows) {
            if (r.getState().get().equals(TicTacToeFieldState.CROSS) && currentPlayer.isUsingCrosses()) {
                returnValue += 1200;
            } else if (r.getState().get().equals(TicTacToeFieldState.CROSS) 
                    && !currentPlayer.isUsingCrosses()) {
                returnValue -= 1200;
            } else if (r.getState().get().equals(TicTacToeFieldState.NOUGHT) 
                    && currentPlayer.isUsingCrosses()) {
                returnValue -= 1200;
            } else if (r.getState().get().equals(TicTacToeFieldState.NOUGHT) && !currentPlayer.isUsingCrosses()) {
                returnValue += 1200;
            }
        }
        
        return Optional.of(returnValue);
    }
}
